SystemOrganization addCategory: #'Pier-Documents'!

!GRPackage class methodsFor: '*pier-documents' stamp: 'lr 2/26/2010 23:09'!
pierDocuments
	^ self new
		name: 'Pier-Documents';
		description: 'Adds support for multiple documents on a single page.';
		addDependency: 'Pier-Model';
		addDependency: 'Pier-Seaside';
		url: #pierAddonsUrl;
		yourself! !

!PRStructure methodsFor: '*pier-documents-accessing' stamp: 'lr 11/13/2007 13:54'!
documents
	"Answer some additional documents to be displayed and edited using the PRDocumentWidget."
	
	^ self propertyAt: #documents ifAbsentPut: [ Dictionary new ]! !

!PRStructure methodsFor: '*pier-documents-accessing' stamp: 'NickAger 2/11/2012 21:35'!
magritteDescription 
	| documents |
	documents := self documents.
	documents isEmpty ifTrue: [ ^ super magritteDescription ].

	^ super magritteDescription copy
		addAll: (documents keys collect: [ :each |
			PRDocumentWidgetDescription new
				parameterName: 'default';
				accessor: (PRDocumentWidgetAccessor on: #documents key: each);
				label: 'Default';
				priority: 120;
				yourself ]);
		yourself! !

MAMemoDescription subclass: #PRDocumentWidgetDescription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Documents'!

!PRDocumentWidgetDescription commentStamp: 'NickAger 2/11/2012 20:14' prior: 0!
I am similar to PRDocumentDescription but I don't redefine #accessor: as I don't want PRDocumentAccessor in the accessor chain as PRDocumentAccessor>>write:to: sets the document #owner: which sets the owner to a PRDocumentWidget rather than the containing PRComponent.!

!PRDocumentWidgetDescription class methodsFor: 'defaults' stamp: 'NickAger 2/12/2012 13:53'!
defaultComponentClasses
	^ Array with: PRDocumentComponent! !

!PRDocumentWidgetDescription class methodsFor: 'defaults' stamp: 'NickAger 2/12/2012 13:53'!
defaultCssClasses
	^ super defaultCssClasses add: 'wiki'; yourself! !

!PRDocumentWidgetDescription class methodsFor: 'defaults' stamp: 'NickAger 2/12/2012 13:52'!
defaultKind
	^ PRDocument! !

!PRDocumentWidgetDescription class methodsFor: 'accessing' stamp: 'NickAger 2/12/2012 13:52'!
label
	^ 'Document'! !

!PRDocumentWidgetDescription methodsFor: 'visiting' stamp: 'NickAger 2/11/2012 20:11'!
acceptMagritte: aVisitor
	aVisitor visitDocumentDescription: self! !

!PRDocumentWidgetDescription methodsFor: 'testing' stamp: 'NickAger 2/11/2012 20:15'!
isDocumentDescription
	^ true! !

!PRDocumentWidgetDescription methodsFor: 'accessing' stamp: 'NickAger 2/11/2012 20:15'!
parserClass
	"Answer the default document parser for the receiver."

	^ PRDocumentParser! !

!PRDocumentWidgetDescription methodsFor: 'accessing' stamp: 'NickAger 2/11/2012 20:15'!
writerClass
	"Answer the default document writer for the receiver."

	^ PRWikiWriter! !

MAChainAccessor subclass: #PRDocumentWidgetAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Documents'!

!PRDocumentWidgetAccessor class methodsFor: 'instance creation' stamp: 'NickAger 2/11/2012 21:34'!
on: selector key: key
	^ self 
		on: (MASelectorAccessor selector: selector)
		accessor:  (MADictionaryAccessor key: key)! !

PRWidget subclass: #PRDocumentWidget
	instanceVariableNames: 'specification default inherited'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Documents'!

!PRDocumentWidget class methodsFor: 'testing' stamp: 'lr 10/17/2007 22:17'!
isAbstract
	^ false! !

!PRDocumentWidget class methodsFor: 'accessing' stamp: 'lr 10/17/2007 22:17'!
label
	^ 'Document'! !

!PRDocumentWidget methodsFor: 'private' stamp: 'NickAger 2/12/2012 16:24'!
buildSpecification
	^ self descriptionDefault
		accessor: (PRDocumentWidgetAccessor on: #documents key: self owner name);
		yourself! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'lr 5/26/2009 17:34'!
children
	^ (PROutgoingReferences on: self document) embeddedComponents
		collect: [ :each | self context componentFor: each ]! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'NickAger 1/22/2012 17:49'!
default
	^ default! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'NickAger 1/22/2012 17:49'!
default: anObject
	default := anObject! !

!PRDocumentWidget methodsFor: 'accessing-descriptions' stamp: 'NickAger 2/11/2012 20:22'!
descriptionDefault
	<magritteDescription>
	^ PRDocumentWidgetDescription new
		parameterName: 'default';
		accessor: #default;
		label: 'Default';
		priority: 120;
		yourself! !

!PRDocumentWidget methodsFor: 'accessing-descriptions' stamp: 'NickAger 1/22/2012 17:07'!
descriptionInherited
	<magritteDescription>
	^ MABooleanDescription new
		parameterName: 'inherited';
		accessor: #inherited;
		label: 'Inherited';
		default: false;
		priority: 110;
		beRequired;
		yourself! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'lr 10/28/2007 12:52'!
document
	^ self documentOf: self context structure! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'lr 11/13/2007 13:37'!
documentOf: aStructure
	"Answer the document to be displayed."

	^ (self specification accessor read: aStructure) ifNil: [
		(self inherited and: [ aStructure parent notNil ])
			ifTrue: [ self documentOf: aStructure parent ]
			ifFalse: [ self default ] ]! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'NickAger 1/22/2012 17:50'!
inherited
	^ inherited! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'NickAger 1/22/2012 17:50'!
inherited: anObject
	inherited := anObject! !

!PRDocumentWidget methodsFor: 'testing' stamp: 'lr 12/2/2009 11:30'!
isEditing
	^ self context command isEdit! !

!PRDocumentWidget methodsFor: 'rendering' stamp: 'lr 12/2/2009 11:30'!
renderContentOn: html
	self isEditing 
		ifTrue: [ self renderEditOn: html ] 
		ifFalse: [ self renderViewOn: html ]! !

!PRDocumentWidget methodsFor: 'rendering' stamp: 'NickAger 1/15/2010 11:15'!
renderEditOn: html
	| id |
	html textArea
		value: self text;
		class: 'wiki-inline';
		id: (id := html nextId);
		callback: [ :value | self text: value ];
		onKeyUp: 'pier_document_widget_update(' , id asJavascript , ')'.
	html document
		addLoadScript: 'pier_document_widget_register(' , id asJavascript , ')'! !

!PRDocumentWidget methodsFor: 'rendering' stamp: 'lr 4/8/2009 20:16'!
renderViewOn: html
	html rendererInstance withinContentDo: [ 
		html rendererInstance
			continue: self document 
			in: self on: html ]! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'lr 12/2/2009 12:42'!
script
	^ self isEditing ifTrue: [ 
'function pier_document_widget_update(id) {
	var source = document.getElementById(id);
	var target = document.getElementById(id + "_copy");
	target.value = source.value;
}
function pier_document_widget_register(id) {
	for (i = 0; i < document.forms.length; i++) {
		var form = document.forms[i];
		if (new RegExp("(^|\\s)magritte(\\s|$)").test(form.className)) {
			var source = document.getElementById(id);
			var target = document.createElement("input");
			target.setAttribute("name", source.name);
			target.setAttribute("id", id + "_copy");
			target.setAttribute("type", "hidden");
			form.appendChild(target);
			pier_document_widget_update(id);
			return;
		}
	}
}' ]! !

!PRDocumentWidget methodsFor: 'accessing' stamp: 'lr 11/13/2007 13:36'!
specification
	"Answer a description representing the receiving document."
	
	^ specification ifNil: [ specification := self buildSpecification ]! !

!PRDocumentWidget methodsFor: 'accessing-editing' stamp: 'lr 11/13/2007 13:48'!
text
	^ self specification 
		toString: (self context command fields 
			at: self specification
			ifAbsentPut: [ 
				self specification accessor
					read: self context structure ])! !

!PRDocumentWidget methodsFor: 'accessing-editing' stamp: 'lr 11/13/2007 14:27'!
text: aString
	| document |
	document := self specification fromString: aString.
	document isNil
		ifFalse: [ document owner: self context structure ].
	self context command fields
		at: self specification
		put: document! !
